
// by Dr. Claudio Soto, 2021/08/16

#include "hmg.ch"

Function Main

LOCAL cCurDir := GETCURRENTFOLDER()
// msginfo(cCurDir)

   DEFINE WINDOW Win_1 ;
      ROW 0 ;
      COL 0 ;
      WIDTH 800 ;
      HEIGHT 600 ;
      TITLE "HMG WebView2";
      ON SIZE RESIZE_WEBVIEW2();
      ON MAXIMIZE RESIZE_WEBVIEW2();
      MAIN


      DEFINE MAIN MENU
         DEFINE POPUP "Navigate"
            MENUITEM "web"    ACTION   NAVIGATE_WEBVIEW2( "http://www.hmgforum.com" )
            MENUITEM "local file"   ACTION   NAVIGATE_WEBVIEW2( "file:///" + cCurDir + "/index.html" )
            SEPARATOR
            MENUITEM "Exec Script"   ACTION   EXECUTESCRIPT_WEBVIEW2( "alert('hello webview');" )
         END POPUP
      END MENU

   END WINDOW

   INIT_WEBVIEW2( Win_1.Handle, "https://srvet.blogspot.com/" )

   Win_1.Center
   Win_1.Activate

Return




#pragma BEGINDUMP
#include "SET_COMPILE_HMG_UNICODE.ch"
#include "HMG_UNICODE.h"
#include <windows.h>
#include "hbapi.h"


#define CINTERFACE
#include "WebView2.h"
#include <functional>

#include <tchar.h>
#include <locale>
#include <string>
#ifndef UNICODE
#include <codecvt>
#endif


class EventHandler : public ICoreWebView2CreateCoreWebView2EnvironmentCompletedHandler,
                     public ICoreWebView2CreateCoreWebView2ControllerCompletedHandler
{
   public:
      EventHandler();
      virtual ~EventHandler();

      EventHandler(const EventHandler&) = delete;
      EventHandler(EventHandler&&) = delete;
      EventHandler& operator=(const EventHandler&) = delete;
      EventHandler& operator=(EventHandler&&) = delete;

      std::function<HRESULT(HRESULT result, ICoreWebView2Environment* created_environment)> EnvironmentCompleted;
      std::function<HRESULT(HRESULT result, ICoreWebView2Controller* controller)> ControllerCompleted;
};


//************ end event.h ********************

template <typename T>
static HRESULT STDMETHODCALLTYPE Null_QueryInterface(T* This, REFIID riid, void** ppvObject) {
   return E_NOINTERFACE;
}

template <typename T>
static ULONG STDMETHODCALLTYPE Null_AddRef(T* This) {
   return 1;
}

template <typename T>
static ULONG STDMETHODCALLTYPE Null_Release(T* This) {
   return 1;
}

static HRESULT STDMETHODCALLTYPE EventInterfaces_EnvironmentCompleted_Invoke(
   ICoreWebView2CreateCoreWebView2EnvironmentCompletedHandler* This, HRESULT result,
   ICoreWebView2Environment* created_environment) {
   return static_cast<EventHandler*>(This)->EnvironmentCompleted(result, created_environment);
}

static ICoreWebView2CreateCoreWebView2EnvironmentCompletedHandlerVtbl EventInterfaces_EnvironmentCompletedHandlerVtbl = {
   Null_QueryInterface,
   Null_AddRef,
   Null_Release,
   EventInterfaces_EnvironmentCompleted_Invoke,
};

static HRESULT STDMETHODCALLTYPE EventInterfaces_ControllerCompleted_Invoke(
   ICoreWebView2CreateCoreWebView2ControllerCompletedHandler* This, HRESULT result,
   ICoreWebView2Controller* controller) {
   return static_cast<EventHandler*>(This)->ControllerCompleted(result, controller);
}

static ICoreWebView2CreateCoreWebView2ControllerCompletedHandlerVtbl EventInterfaces_ControllerCompletedHandlerVtbl = {
   Null_QueryInterface,
   Null_AddRef,
   Null_Release,
   EventInterfaces_ControllerCompleted_Invoke,
};

EventHandler::EventHandler() :
   ICoreWebView2CreateCoreWebView2EnvironmentCompletedHandler { &EventInterfaces_EnvironmentCompletedHandlerVtbl },
   ICoreWebView2CreateCoreWebView2ControllerCompletedHandler  { &EventInterfaces_ControllerCompletedHandlerVtbl  } {
}

EventHandler::~EventHandler() = default;


//***** end event.cpp ***************


ICoreWebView2* webview2 = nullptr;
ICoreWebView2Controller* controller = nullptr;
EventHandler handler{};
HWND hWnd;
WCHAR * szURL;
BOOL isReady = FALSE;

std::wstring TStrToWStr(LPCTSTR str) {
   #ifdef UNICODE
      return std::wstring(str);
   #else
      std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>> converter;
      return converter.from_bytes(str);
   #endif
}


void GetDataPath(LPTSTR szOut, DWORD nSize) {
   TCHAR szExePath[MAX_PATH];
   GetModuleFileName(nullptr, szExePath, _countof(szExePath));

   LPTSTR szExeName = _tcsrchr(szExePath, TEXT('\\'));
   szExeName = szExeName ? szExeName + 1 : szExePath;

   TCHAR szAppData[MAX_PATH];
   GetEnvironmentVariable(TEXT("AppData"), szAppData, _countof(szAppData));

   _tcsncpy(szOut, szAppData, nSize);
   _tcsncat(szOut, TEXT("\\"), nSize);
   _tcsncat(szOut, szExeName, nSize);
}


void FatalError(LPCTSTR format, ...) {
   MessageBox(nullptr, format, TEXT("My WebView2 ERROR"), MB_OK | MB_ICONSTOP);
   ExitProcess(1);
}


void ResizeWebView2(HWND hWnd) {
   if (!controller)
      return;
   RECT bounds;
   GetClientRect(hWnd, &bounds);
   controller->lpVtbl->put_Bounds(controller, bounds);
}


HRESULT EnvironmentCompleted_myInvoke(HRESULT result, ICoreWebView2Environment* created_environment) {
   if (FAILED(result))
      FatalError(TEXT("Failed to create environment?"));
   created_environment->lpVtbl->CreateCoreWebView2Controller(created_environment, hWnd, &handler);
   return S_OK;
};


HRESULT STDMETHODCALLTYPE ControllerCompleted_myInvoke(HRESULT result, ICoreWebView2Controller* new_controller) {
   if (FAILED(result))
      FatalError(TEXT("Failed to create controller?"));
   controller = new_controller;
   controller->lpVtbl->AddRef(controller);
   controller->lpVtbl->get_CoreWebView2(controller, &webview2);
   webview2->lpVtbl->AddRef(webview2);

   ICoreWebView2Settings* Settings;
   webview2->lpVtbl->get_Settings(webview2, &Settings);
   Settings->lpVtbl->put_IsScriptEnabled(Settings, TRUE);
   Settings->lpVtbl->put_AreDefaultScriptDialogsEnabled(Settings, TRUE);
   Settings->lpVtbl->put_IsWebMessageEnabled(Settings, TRUE);
   Settings->lpVtbl->put_AreDevToolsEnabled(Settings, FALSE);
   // Settings->lpVtbl->put_AreDefaultContextMenusEnabled(Settings, FALSE);
   Settings->lpVtbl->put_IsStatusBarEnabled(Settings, FALSE);

   webview2->lpVtbl->Navigate(webview2, szURL );
   // webview2->lpVtbl->Navigate(webview2, L"https://srvet.blogspot.com/");

   isReady = TRUE;
   ResizeWebView2(hWnd);
   return S_OK;
};


void runWebView2(){
   // Set up some event handlers.
   handler.EnvironmentCompleted = &EnvironmentCompleted_myInvoke;
   handler.ControllerCompleted  = &ControllerCompleted_myInvoke;

   // Get data path.
   TCHAR szDataPath[MAX_PATH];
   GetDataPath(szDataPath, _countof(szDataPath));

   // Create a webview
   HRESULT result = CreateCoreWebView2EnvironmentWithOptions( nullptr, TStrToWStr(szDataPath).c_str(), nullptr, &handler );
   if (FAILED(result))
      FatalError(TEXT("Call to CreateCoreWebView2EnvironmentWithOptions failed!"));
}


HB_FUNC( INIT_WEBVIEW2 )
{
   hWnd = (HWND) hb_parnl( 1 );
   szURL = HMG_CHAR_TO_WCHAR( hb_parc( 2 ) );
   runWebView2();
}


HB_FUNC( RESIZE_WEBVIEW2 )
{
   ResizeWebView2( hWnd );
}


HB_FUNC( NAVIGATE_WEBVIEW2 )
{
   WCHAR * szURL = HMG_CHAR_TO_WCHAR( hb_parc( 1 ) );
   if( isReady )
      webview2->lpVtbl->Navigate( webview2, szURL );
}


HB_FUNC( EXECUTESCRIPT_WEBVIEW2 )
{
   WCHAR * szScript = HMG_CHAR_TO_WCHAR( hb_parc( 1 ) );
   if( isReady )
      webview2->lpVtbl->ExecuteScript( webview2, szScript, nullptr );
}


#pragma ENDDUMP